#!/usr/bin/env python

__all__ = []

import re

from a2p2.vlti.instrument import OBConstraints
from a2p2.vlti.instrument import OBTarget
from a2p2.vlti.instrument import TSF
from a2p2.vlti.instrument import VltiInstrument

HELPTEXT = """
Please define PIONIER instrument help in a2p2/vlti/pionier.py
"""


class Pionier(VltiInstrument):

    def __init__(self, facility):
        VltiInstrument.__init__(self, facility, "PIONIER")

    def checkOB(self, ob, p2container=None):
        ui = self.ui

        instrumentConfiguration = ob.instrumentConfiguration
        BASELINE = ob.interferometerConfiguration.stations
        # Compute tel = UT or AT
        # TODO move code into common part
        if "U" in BASELINE:
            tel = "UT"
        else:
            tel = "AT"

        instrumentMode = instrumentConfiguration.instrumentMode

        # Retrieve GRISM or FREE info from instrumentMode
        for disp in self.getRange("PIONIER_acq.tsf", "INS.DISP.NAME"):
            if disp in instrumentMode[0:len(disp)]:
                ins_disp = disp

        for observationConfiguration in self.getSequence(ob):

            # create keywords storage objects
            acqTSF = TSF(self, "PIONIER_acq.tsf")

            # alias for PIONIER_obs_calibrator.tsf and
            # PIONIER_obs_science.tsf")
            kappaTSF = TSF(self, "PIONIER_gen_cal_kappa.tsf")
            darkTSF = TSF(self, "PIONIER_gen_cal_dark.tsf")


            obConstraints = OBConstraints(self)

            # set common properties
            acqTSF.INS_DISP_NAME = ins_disp

            if 'SCIENCE' in observationConfiguration.type:
                OBJTYPE = 'SCIENCE'
                obsTSF = TSF(self, "PIONIER_obs_science.tsf")
            else:
                OBJTYPE = 'CALIBRATOR'
                obsTSF = TSF(self, "PIONIER_obs_calibrator.tsf")

            scienceTarget = observationConfiguration.SCTarget

            obTarget = OBTarget(self, scienceTarget)

            # Set baseline  interferometric array code (should be a keywordlist)
            acqTSF.ISS_BASELINE = [self.getBaselineCode(ob)]

            self.checkIssVltiType(acqTSF)

            # define some default values
            DIAMETER = float(self.get(scienceTarget, "DIAMETER", 0.0))
            VIS = 1.0  # FIXME

            # Retrieve Fluxes
            TEL_COU_MAG = self.getFlux(scienceTarget, "V")
            acqTSF.ISS_IAS_HMAG = self.getFlux(scienceTarget, "H")

            # setup some default values, to be changed below
            TEL_COU_GSSOURCE = 'SCIENCE'  # by default
            GSRA = '00:00:00.000'
            GSDEC = '00:00:00.000'

            # initialize FT variables (must exist)

            # AO target
            aoTarget = ob.get(observationConfiguration, "AOTarget")
            if aoTarget != None:
                AONAME = aoTarget.name
                TEL_COU_GSSOURCE = 'SETUPFILE'  # since we have an AO
                # TODO check if AO coords should be required by template
                # AORA, AODEC  = self.getCoords(aoTarget,
                # requirePrecision=False)
                acqTSF.TEL_COU_PMA, acqTSF.TEL_COU_PMD = self.getPMCoords(
                    aoTarget)

            # Guide Star
            gsTarget = ob.get(observationConfiguration, 'GSTarget')
            if gsTarget != None:
                TEL_COU_GSSOURCE = 'SETUPFILE'  # since we have an GS
                GSRA, GSDEC = self.getCoords(gsTarget, requirePrecision=False)
                # no PMRA, PMDE for GS !!
                TEL_COU_MAG = float(gsTarget.FLUX_V)

            acqTSF.TEL_COU_GSSOURCE = TEL_COU_GSSOURCE
            acqTSF.TEL_COU_ALPHA = GSRA
            acqTSF.TEL_COU_DELTA = GSDEC
            acqTSF.TEL_COU_MAG = round(TEL_COU_MAG, 3)

            # LST interval
            try:
                obsConstraint = observationConfiguration.observationConstraints
                LSTINTERVAL = obsConstraint.LSTinterval
            except:
                LSTINTERVAL = None

            # Constraints
            obConstraints.name = 'Aspro-created constraints'
            skyTransparencyMagLimits = {"AT": 3, "UT": 5}
            if acqTSF.ISS_IAS_HMAG < skyTransparencyMagLimits[tel]:
                obConstraints.skyTransparency = 'Variable, thin cirrus'
            else:
                obConstraints.skyTransparency = 'Clear'

            # FIXME: error (OB): "Phase 2 constraints must closely follow what was requested in the Phase 1 proposal.
            # The seeing value allowed for this OB is >= java0x0 arcsec."
            # FIXME REPLACE SEEING THAT IS NO MORE SUPPORTED
            # obConstraints.seeing = 1.0
            # FIXME: default values NOT IN ASPRO!
            # constaints.airmass = 5.0
            # constaints.fli = 1

            # compute dit, ndit, nexp

            # and store computed values in obsTSF
            obsTSF.SEQ_NEXPO = 5
            # obsTSF.NSCANS = 100
            # kappaTSF.SEQ_DOIT=False
            # darkTSF.SEQ_DOIT=True

            # then call the ob-creation using the API if p2container exists.
            if p2container == None:
                ui.addToLog(obTarget.name +
                            " ready for p2 upload (details logged)")
                ui.addToLog(obTarget, False)
                ui.addToLog(obConstraints, False)
                ui.addToLog(acqTSF, False)
                ui.addToLog(obsTSF, False)
                ui.addToLog(kappaTSF, False)
                ui.addToLog(darkTSF, False)
            else:
                self.createOB(p2container, obTarget, obConstraints, OBJTYPE, instrumentMode,
                              LSTINTERVAL, [acqTSF, obsTSF, kappaTSF, darkTSF])

    def formatDitTable(self):
        buffer = ' No dit table in use \n'

#        ditTable = self.getDitTable()
#        buffer = '   Tel | Spec |  Pol  |     H   | DIT(s)\n'
#        buffer += '--------------------------------------------------------\n'
#        for tel in ['AT']:
#           for spec in ['GRISM', 'FREE']:
#                for pol in ['IN', 'OUT']:
#                    for i in range(len(ditTable[tel][spec][pol]['DIT'])):
#                        buffer += ' %3s | %4s | %3s | %2s |' % (tel,
#                                                                spec, pol, tel)
#                        buffer += ' %4.1f <K<= %3.1f | %4.1f' % (ditTable[tel][spec][pol]['MAG'][i],
#                                                                 ditTable[tel][spec][
#                                                                     pol]['MAG'][i + 1],
#                                                                 ditTable[tel][spec][pol]['DIT'][i])
#                        buffer += "\n"
#            Hut = ditTable[tel]['Hut']
        return buffer



#!/usr/bin/env python

__all__ = []

from a2p2.chara.gui import CharaUI
from a2p2.facility import Facility
import requests
import logging
import traceback

HELPTEXT = "TODO update this HELP message in a2p2/chara/facility.py"

logger = logging.getLogger(__name__)

class CharaFacility(Facility):

    def __init__(self, a2p2client):
        Facility.__init__(self, a2p2client, "CHARA", HELPTEXT)
        self.charaUI = CharaUI(self)
        self.connected2OB2 = False
        self.validQueueServer = None

    def processOB(self, ob):
        if not ob:
            return

        self.a2p2client.ui.addToLog(
            "OB received for '" + self.facilityName + "' interferometer")
        # show ob dict for debug
        self.a2p2client.ui.addToLog(str(ob), False)

        # performs operation
        self.consumeOB(ob)

        # give focus on last updated UI
        self.a2p2client.ui.showFacilityUI(self.charaUI)

    def checkServer(self):
        if self.validQueueServer:
            # recheck ?
            return True
        else:
            # search and test servers from preferences
            queueServers=self.a2p2client.preferences.getCharaQueueServer()
            msg=""
            if queueServers :
                for queueServer in queueServers:
                    logger.debug(f'Trying to send OB on queuserver : {queueServer}')
                    try:
                        c=requests.get(queueServer, timeout=3)
                        msg+=f"Connection succeeded on OB server : {c.json()}\n"
                        self.validQueueServer = queueServer
                        break
                    except:
                        msg+=f"Connection failed on {queueServer}\n"

            self.a2p2client.ui.addToLog(msg)
            self.charaUI.display(msg)

            return self.validQueueServer

    def consumeOB(self, ob):
        if self.checkServer():
            try:
                r = requests.post(self.validQueueServer, json=ob.as_dict(), timeout=3)
                msg = ""
                msg+=f"OB sent to remote server queue : {r}"
                self.connected2OB2 = True
                self.a2p2client.ui.addToLog(msg)
                self.charaUI.display(msg)
            except:
                print(traceback.format_exc())
                msg=f"Can't send OB to the '{self.validQueueServer}' queue server, please relaunch it or try again for a new one."
                self.validQueueServer = None
                self.a2p2client.ui.addToLog(msg)
                self.charaUI.display(msg)
        else:
            msg=f"Can't find any queue server, please launch it, edit your preferences or check your ssh port forwarding"

        # display OB
        self.charaUI.displayOB(ob)





#!/usr/bin/env python

__all__ = []

import re

from a2p2.vlti.instrument import OBConstraints
from a2p2.vlti.instrument import OBTarget
from a2p2.vlti.instrument import TSF
from a2p2.vlti.instrument import VltiInstrument
import logging

HELPTEXT = """
    Please define MATISSE instrument help in a2p2/vlti/matisse.py
"""


class Matisse(VltiInstrument):

    def __init__(self, facility):
        # MATISSE_LM" MATISSE_N
        VltiInstrument.__init__(self, facility, "MATISSE_LM")

    def checkOB(self, ob, p2container=None):
        ui = self.ui

        instrumentConfiguration = ob.instrumentConfiguration
        BASELINE = ob.interferometerConfiguration.stations
        # Compute tel = UT or AT
        # TODO move code into common part
        if "U" in BASELINE:
            tel = "UT"
        else:
            tel = "AT"

        instrumentMode = instrumentConfiguration.instrumentMode
        fringeTrackerMode = "GRA4MAT" in instrumentConfiguration.fringeTrackerMode
        if fringeTrackerMode:
            tsfSuffix = "_ft"
            ui.addToLog("OB requested with GRA4MAT")
        else:
            tsfSuffix = ""
            ui.addToLog("OB requested without GRA4MAT")

        for observationConfiguration in self.getSequence(ob):

            # create keywords storage objects
            acqTSF = TSF(self, f"MATISSE_img_acq{tsfSuffix}.tsf")  # or .tsfx?
            obsTSF = TSF(self, f"MATISSE_hyb_obs{tsfSuffix}.tsf")

            obConstraints = OBConstraints(self)

            if 'SCIENCE' in observationConfiguration.type:
                obsTSF.DPR_CATG = 'SCIENCE'  # Default OB are CALIB
            else:
                obsTSF.DPR_CATG = 'CALIB'

            scienceTarget = observationConfiguration.SCTarget
            obTarget = OBTarget(self, scienceTarget)

            # Set baseline  interferometric array code (should be a keywordlist)
            acqTSF.ISS_BASELINE = [self.getBaselineCode(ob)]

            self.checkIssVltiType(acqTSF)

            # define some default values
            VIS = 1.0  # FIXME

            # Retrieve Fluxes
            acqTSF.SEQ_TARG_FLUX_L = self.getFlux(scienceTarget, "L_JY")
            acqTSF.SEQ_TARG_FLUX_N = self.getFlux(scienceTarget, "N_JY")
            acqTSF.SEQ_TARG_MAG_K = self.getFlux(scienceTarget, "K")
            # H is only present in _ft template
            if fringeTrackerMode:
                try:
                    acqTSF.SEQ_TARG_MAG_H = self.getFlux(scienceTarget, "H")
                except:
                    ui.addToLog("H mag is missing. default value will be used",
                                displayString=True, level=logging.WARNING)

            # TODO : make AO & GS more consistent ( on Aspro2 side ? )
            # AO target
            aoTarget = ob.get(observationConfiguration, "AOTarget")
            if aoTarget != None:
                acqTSF.COU_AG_GSSOURCE = 'SETUPFILE'  # since we have an AO
                # TODO check if AO coords should be required by template
                # AORA, AODEC  = self.getCoords(aoTarget,
                # requirePrecision=False)
                acqTSF.COU_AG_PMA, acqTSF.COU_AG_PMD = self.getPMCoords(
                    aoTarget)

            # Guide Star
            gsTarget = ob.get(observationConfiguration, 'GSTarget')
            if gsTarget != None:
                acqTSF.COU_AG_GSSOURCE = 'SETUPFILE'  # since we have a GS
                # special hack waiting for uniform way
                alpha, delta = self.getCoords(gsTarget, requirePrecision=False)
                acqTSF.COU_AG_ALPHA, acqTSF.COU_AG_DELTA = alpha.replace(
                    ":", ""), delta.replace(":", "")
                acqTSF.COU_AG_PMA, acqTSF.COU_AG_PMD = self.getPMCoords(
                    gsTarget)

                # no PMRA, PMDE for GS ??

                acqTSF.COU_GS_MAG = self.getFlux(gsTarget, "V")
                try:
                    if "AT" in tel:  # try better band observing on UT
                        acqTSF.COU_GS_MAG = self.getFlux(gsTarget, "G")
                except:
                    ui.addToLog(
                        "G mag can't be retreived for AT on the guide star. Use V instead")
            else:
                acqTSF.COU_GS_MAG = self.getFlux(scienceTarget, "V")
                try:
                    if "AT" in tel:  # try better band observing on UT
                        acqTSF.COU_GS_MAG = self.getFlux(scienceTarget, "G")
                except:
                    ui.addToLog(
                        "G mag can't be retreived for AT on the science star. Use V instead")

            # LST interval
            try:
                obsConstraint = observationConfiguration.observationConstraints
                LSTINTERVAL = obsConstraint.LSTinterval
            except:
                LSTINTERVAL = None

            # Constraints
            obConstraints.name = 'Aspro-created constraints'
            skyTransparencyMagLimits = {"AT": 3, "UT": 5}
            #            if acqTSF.IAS.HMAG < skyTransparencyMagLimits[tel]:
            #                obConstraints.skyTransparency = 'Variable, thin cirrus'
            #            else:
            #                obConstraints.skyTransparency = 'Clear'

            obConstraints.skyTransparency = 'Clear'
            # FIXME: error (OB): "Phase 2 constraints must closely follow what was requested in the Phase 1 proposal.
            # The seeing value allowed for this OB is >= java0x0 arcsec."
            # FIXME REPLACE SEEING THAT IS NO MORE SUPPORTED
            # obConstraints.seeing = 1.0
            # baseline not in instrumecnstraints obConstraints.baseline = BASELINE.replace(' ', '-')
            # FIXME: default values NOT IN ASPRO!
            # constaints.airmass = 5.0
            # constaints.fli = 1
            # and store computed values in obsTSF

            # then call the ob-creation using the API if p2container exists.
            if p2container == None:
                ui.addToLog(obTarget.name +
                            " ready for p2 upload (details logged)")
                ui.addToLog(obTarget, False)
                ui.addToLog(obConstraints, False)
                ui.addToLog(acqTSF, False)
                ui.addToLog(obsTSF, False)
            else:
                self.createOB(p2container, obTarget, obConstraints, obsTSF.DPR_CATG, instrumentMode,
                                     LSTINTERVAL, [acqTSF, obsTSF])

    def formatDitTable(self):
        #    fluxTable = self.getDitTable()
        buffer = ' No dit table in use \n'
        # buffer = '   Tel | Spec |  spec band  | Flux (Jy)    | tau(ms)\n'
        #    buffer += '--------------------------------------------------------\n'
        #    for tel in ['AT']:
        #        for spec in ['Low','Med']:
        #            for band in  ['L','M','N']:
        #                for i in range(len(fluxTable[tel][spec][band]['Flux'])):
        #                    buffer += ' %3s | %4s | %3s | %2s |' % ( tel,
        #                        spec, band, tel)
        #                    buffer += ' %4.1f <K<= %3.1f | %4.1f' % (fluxTable[tel][spec][pol]['MAG'][i],
        #                                                             fluxTable[tel][spec][
        #                        pol]['MAG'][i + 1],
        #                        fluxTable[tel][spec][pol]['DIT'][i])
        #                    buffer += "\n"

        return buffer



from setuptools import setup, find_packages

with open('README.rst') as README:
    long_description = README.read()
    description = long_description[
        :long_description.index('Description')].split("*")[1].strip()

versions = {}
with open("a2p2/version.py") as fp:
    exec(fp.read(), versions)

setup(
    version=versions['__version__'],
    description=description
# not yet moved to pyproject.toml
#    packages=find_packages(),
#    include_package_data=True,
#    keywords='observation preparation tool optical-interferometry p2 samp'
)



#!/usr/bin/env python

__all__ = []

import sys
import logging
from turtle import bgcolor

from a2p2.gui import FacilityUI

from tkinter import ttk
from tkinter import *
from tkinter.ttk import *

logger = logging.getLogger(__name__)


class VltiUI(FacilityUI):

    def __init__(self, a2p2client):
        logger.info("init VltiUI")
        FacilityUI.__init__(self, a2p2client)

        #self.container = Frame(self, bd=3, relief=SUNKEN)
        self.container = Frame(self, relief=SUNKEN)
        self.container.grid_rowconfigure(0, weight=1)
        self.container.grid_columnconfigure(0, weight=1)

        self.loginFrame = LoginFrame(self)
        self.loginFrame.grid(row=0, column=0, sticky="nsew")

        self.treeFrame = TreeFrame(self)
        self.treeFrame.grid(row=0, column=0, sticky="nsew")
        self.tree = self.treeFrame.tree

        self.container.pack(fill=BOTH, expand=True)
        self.treeItemToRuns = {}
        self.treeItemToP2Items = {}

        self.ob = None

    def showLoginFrame(self, ob):
        self.ob = ob
        self.addToLog(
            "Sorry, your %s OB can't be submitted, please log in first, select container and press submit button." %
            (ob.instrumentConfiguration.name))
        self.loginFrame.tkraise()

    def isBusy(self):
        self.tree.configure(selectmode='none')

    def isIdle(self):
        self.tree.configure(selectmode='browse')

    def showTreeFrame(self, ob=None):
        if ob:
            instrum = ob.instrumentConfiguration.name
        else:
            instrum = "incoming"
        self.addToLog(
            "Please select a runId in ESO P2 database to process %s OB" % instrum)
        self.treeFrame.tkraise()

    def fillTree(self, runs):
        logger.debug(f"Start filling tree with {len(runs)} runs")
        if len(runs) == 0:
            self.ShowErrorMessage(
                "No Runs defined, impossible to program ESO's P2 interface.")
            return
        # cleanup old entries if any
        for item in self.treeItemToRuns:
            try:
                self.tree.delete(item)
            except:
                pass
        self.treeItemToRuns = {}

        for run in runs:
            if self.facility.hasSupportedInsname(run['instrument']):
                cid = run['containerId']
                self._insertInTree('', run['progId'], cid, run, run)

    def updateTree(self, run, containerId):
        logger.debug(
            f"updateTree with run : {run}\n and containerId: {containerId}")
        items, _ = self.facility.api.getItems(containerId)
        logger.debug(f"sub items are {items}")
        logger.debug(f"item keys in tree: {self.treeItemToP2Items.keys()}")
        for item in items:
            if item['itemType'] in ['Folder', 'Concatenation', 'Group']:
                if str(item['containerId']) in self.treeItemToP2Items.keys():
                    logger.debug(f"Item {item['containerId']} already in tree")
                else:
                    logger.debug(f"adding in tree : {item}")
                    self._insertInTree(
                        containerId, item['name'], item['containerId'], run, item)
            else:
                logger.debug(f"not inserted in tree : {item}")
        # TODO get children of our tree and remove part non present in items

    def selectTreeItem(self, containerId):
        item = str(containerId)
        self.tree.selection_set(item)
        self.tree.see(item)
        self.facility.containerInfo.store(
            self.treeItemToRuns[item], self.treeItemToP2Items[item])

    def _insertInTree(self, parentContainerID, name, containerID, run, item):
        instrument = run['instrument']
        if run != item:
            label = item['itemType']
        else:
            # {"runId":60900301,"progId":"60.A-9003(B)","title":"Tutorial account","period":60,
            # "scheduledPeriod":60,"mode":"VM","instrument":"FORS2","telescope":"UT1","ipVersion":104.26,
            # "isToO":false,"owned":true,"delegated":false,"itemCount":0,"containerId":2587672,
            # "pi":{"emailAddress":"52052@nodomain.net","firstName":"Phase 1/2 Tutorial","lastName":"Account"},
            # "observingConstraints":{"seeing":2.0}}
            label = "%s Run (IP %s)" % (run['mode'], run['ipVersion'])

        e = self.tree.insert(parentContainerID, 'end', containerID,
                             text=name, values=(run['instrument'], label))
        self.treeItemToRuns[e] = run
        self.treeItemToP2Items[e] = item

        # add a dummy child so we can show to the user he has to walk into the tree
        if run['itemCount'] > 0:
            self.tree.insert(containerID, 'end',
                             f"{containerID}_fillme", values=[])
        # return new tree item
        return e

    def on_tree_selection_changed(self, selection):
        logger.debug(f"tree selection change: {selection}")
        curItem = self.tree.focus()
        ret = self.tree.item(curItem)
        if len(ret['values']) > 0:
            self.facility.containerInfo.store(
                self.treeItemToRuns[curItem], self.treeItemToP2Items[curItem])
            self.facility.tryLastOB()

    def on_tree_open(self, selection):
        logger.debug(f"tree open: {selection}")
        curItem = self.tree.focus()
        curChildren = self.tree.get_children(curItem)
        firstChild = curChildren[0]
        if not firstChild in self.treeItemToRuns.keys():
            self.tree.delete(firstChild)

            parentRun = self.treeItemToRuns[curItem]
            parentContainer = self.treeItemToP2Items[curItem]
            parentContainerID = parentContainer['containerId']
            self.updateTree(parentRun, parentContainerID)

    def showOBConfirmationButtons(self, ob=None, showAcqButtons=False):
        if not (ob):
            self.treeFrame.enable_buttons(False, showAcqButtons)
            return
        else:
            self.treeFrame.enable_buttons(True, showAcqButtons)

    def getIssVltitypeVars(self):
        return self.treeFrame.issVltiTypeCheckboxesVars

    def getAcquisitionType(self):
        return self.treeFrame.acquisitionType.get()


class TreeFrame(Frame):

    def __init__(self, vltiUI):
        Frame.__init__(self, vltiUI.container)
        self.vltiUI = vltiUI
        subframe = Frame(self)

        treeframe = Frame(subframe)
        self.tree = ttk.Treeview(
            treeframe, columns=('Project ID'), selectmode='browse')  # , 'instrument'))#, 'folder ID'))
        ysb = ttk.Scrollbar(
            treeframe, orient='vertical', command=self.tree.yview)
        xsb = ttk.Scrollbar(
            treeframe, orient='horizontal', command=self.tree.xview)

        self.tree.configure(yscroll=ysb.set, xscroll=xsb.set)
        self.tree.heading('#0', text='Project ID', anchor='w')
        self.tree.heading('#1', text='Instrument', anchor='w')
        self.tree.heading('#2', text='Container type', anchor='w')
        self.tree.bind(
            '<<TreeviewOpen>>', self.vltiUI.on_tree_open)
        self.tree.bind(
            '<<TreeviewSelect>>', self.vltiUI.on_tree_selection_changed)

        # grid layout does not expand and fill all area then move to pack
        #       self.tree.grid(row=0, column=0, sticky='nsew')
        #       ysb.grid(row=0, column=1, sticky='ns')
        #       xsb.grid(row=1, column=0, sticky='ew')
        self.tree.pack(side=LEFT, fill=BOTH, expand=True)
        ysb.pack(side=RIGHT, fill="y")
        # xsb.pack(side=BOTTOM, fill="y") # will probably require to add
        # another 2 frames
        treeframe.pack(side=TOP, fill=BOTH, expand=True)

        # Control button Panel to abort /submit or provide more information to the current OB
        buttonframe = Frame(subframe)

        cindex = 0


        acquisitionFrame = Frame(buttonframe)

        self.acquisitionType = StringVar()
        self.acquisitionType.set("onaxis") # initialiser
        self.r1 = Radiobutton(acquisitionFrame, text="onaxis", variable=self.acquisitionType, value="onaxis")
        self.r2 = Radiobutton(acquisitionFrame, text="offaxis", variable=self.acquisitionType, value="offaxis")
        self.r3 = Radiobutton(acquisitionFrame, text="wide", variable=self.acquisitionType, value="wide")

        acquisitionFrame.grid(row=0, column=cindex)
        cindex += 1


        label = Label(buttonframe, text="|")
        label.grid(row=0, column=cindex)
        cindex += 1

        psection = "p2.iss.vltitype"
        pvltitypes = self.vltiUI.facility.a2p2client.preferences.getConfigKeys(
            psection)

        self.issVltiTypeCheckboxesVars = {}
        for isvt in self.vltiUI.facility.getIssVltiTypes():
            boolv = BooleanVar()
            self.issVltiTypeCheckboxesVars[isvt]=boolv
            boolv.set( not pvltitypes or len(pvltitypes)==0 or isvt in pvltitypes )
            logger.debug(f"Set preference value for {isvt} : {boolv.get()}")
            cb = Checkbutton(buttonframe, text=isvt, variable=boolv, onvalue=True, offvalue=False)
            cb.grid(row=0, column=cindex)
            cindex += 1

        self.abortButton = Button(buttonframe, text="Abort this OB",
                                  command=self.on_abortbutton_clicked, state=DISABLED)
        style = ttk.Style()
        sButtonStyle="Custom.TButton"
        ttk.Style().configure(sButtonStyle, background="green")
        self.submitButton = Button(
            buttonframe, text="Submit", command=self.on_submitbutton_clicked, state=DISABLED,
            style=sButtonStyle)
        self.abortButton.grid(row=0, column=cindex)
        cindex += 1
        self.submitButton.grid(row=0, column=cindex)
        buttonframe.pack(side=RIGHT)

        subframe.pack(side=TOP, fill=BOTH, expand=True)

    def enable_buttons(self, enableActions, showAcqButtons=False):
        if showAcqButtons and enableActions:
            self.r1.pack(side=LEFT)
            self.r2.pack(side=LEFT)
            self.r3.pack(side=LEFT)
        else:
            self.r1.pack_forget()
            self.r2.pack_forget()
            self.r3.pack_forget()

        if enableActions:
            flag = NORMAL
        else:
            flag = DISABLED
        self.submitButton["state"] = flag
        self.abortButton["state"] = flag

        # leave abort button always active to let user remove in case of error
        if self.vltiUI.facility.hasOB():
            self.abortButton["state"] = NORMAL

    def on_submitbutton_clicked(self):
        self.vltiUI.facility.popOB()
        self.enable_buttons(False)

    def on_abortbutton_clicked(self):
        self.vltiUI.facility.popOB(ignore=True)
        self.enable_buttons(False)


class LoginFrame(Frame):

    def __init__(self, vltiUI):
        Frame.__init__(self, vltiUI.container)
        self.vltiUI = vltiUI

        prefs = vltiUI.a2p2client.preferences

        self.loginframe = LabelFrame(
            self, text="login (ESO USER PORTAL or demo account)")

        self.username_label = Label(self.loginframe, text="USERNAME")
        self.username_label.pack()
        self.username = StringVar()
        self.username.set(prefs.getP2Username())
        self.username_entry = Entry(
            self.loginframe, textvariable=self.username)
        self.username_entry.pack()

        self.password_label = Label(self.loginframe, text="PASSWORD")
        self.password_label.pack()
        self.password = StringVar()
        self.password.set(prefs.getP2Password())
        self.password_entry = Entry(
            self.loginframe, textvariable=self.password, show="*")
        self.password_entry.pack()

        self.loginbutton = Button(
            self.loginframe, text="LOG IN", command=self.on_loginbutton_clicked)
        self.loginbutton.pack()

        self.loginframe.pack()
        self.pack(side=TOP, fill=BOTH, expand=True)

    def on_loginbutton_clicked(self):
        self.vltiUI.facility.connectAPI(
            self.username.get(), self.password.get(), self.vltiUI.ob)


# TODO move into a common part
def getFolders(p2api, containerId):
    folders = []
    items, _ = p2api.getItems(containerId)
    for item in items:
        if item['itemType'] in ['Folder', 'Concatenation']:
            folders.append(item)
    return folders



#!/usr/bin/env python

__all__ = []

import os
from tkinter.constants import TRUE
import traceback
import logging
import re

import p2api

from a2p2.facility import Facility
from a2p2.vlti.gui import VltiUI

MODE_SM = 'SM'

ITEMTYPE_CONCATENATION = 'Concatenation'

JMMC_LOGIN = 'jmmc'

TUTORIAL_LOGIN = '52052'
PRODUCTION_APITYPE = 'production'
DEMO_APITYPE = 'demo'

# TODO handle a period subdirectory
CONFDIR = "conf"

# Look for configuration files in the same level directory as this module/conf/
try:
    _confdir = os.path.join(os.path.dirname(__file__), CONFDIR)
except NameError:
    _confdir = CONFDIR
if os.path.isdir(_confdir):
    CONFDIR = _confdir
elif not os.path.isdir(CONFDIR):
    raise RuntimeError("can't find conf directory (%r)" % (CONFDIR,))

HELPTEXT = """
ESO's P2 repository for Observing Blocks (OBs):


Send configuration from Aspro:
- In ASPRO, have an object, or an object selected, and check that all important informations (magnitudes, but also Instrument and Fringe Tracker Modes, eventually hour angles), are correctly set.
- In menu "Interop" select "Send Obs. blocks to A2p2"
- Block(s) are created and put in the P2 repository.
- If the source had one or more calibrators, blocks are created for them too.
- For each block submitted, a report is produced. Warnings are usually not significant.
- For more than 1 object sent, a <b>folder</b> containing the two or more blocks is created. In the absence of availability of grouping OBs (like for CAL-SCI-CAL) provided by ESO, this is the closets we can do.
- All the new OBs and folders will be available on p2web at https://www.eso.org/p2

On your first valid VLTI's OB, A2p2 will ask for your ESO User Portal credentials to interact with P2 using it's API. You can follow actions and organize more in details your OB on https://www.eso.org/p2 . Please use prefilled values of the login panel for testing purpose and follow ESO database from https://www.eso.org/p2demo/
"""
HELPTEXT += "Config files loaded from " + CONFDIR

logger = logging.getLogger(__name__)

class VltiFacility(Facility):

    def __init__(self, a2p2client):
        Facility.__init__(self, a2p2client, VltiFacility.getName(), HELPTEXT)
        self.ui = VltiUI(self)

        # Instanciate instruments
        # TODO complete list and make it more object oriented
        from a2p2.vlti.gravity import Gravity
        gravity = Gravity(self)
        from a2p2.vlti.pionier import Pionier
        pionier = Pionier(self)
        from a2p2.vlti.matisse import Matisse
        matisse = Matisse(self)

        # complete help
        for i in self.getSupportedInstruments():
            self.facilityHelp += "\n" + i.getHelp()

        self.connected = False
        # set to demo by default (may change in connectAPI)
        self.apitype = 'demo'
        self.containerInfo = P2Container(self)

        # will store later : name for status info, api
        self.username = None
        self.api = None

        # store ob to insert after container selection if not done previously
        self.ob_to_confirm=[]
        self.ob_to_submit=[]


    def getName():
        return "VLTI"

    def autologin(self):
        self.ui.addToLog("\nAutologin into P2 API please wait...\n")
        self.ui.loginFrame.on_loginbutton_clicked()
        self.a2p2client.ui.showFacilityUI(self.ui)
        self.ui.showTreeFrame()

    def processOB(self, ob):
        # process last accepted OB if ob is None
        if not ob:
            self.processLastAcceptedOB()
            return

        # give focus on last updated UI
        self.a2p2client.ui.showFacilityUI(self.ui)
        # show ob dict for debug
        self.ui.addToLog(str(ob), False)

        # OB is checked and submitted by instrument
        period = ob.getPeriod()
        self.ui.addToLog(f"Request received for OB creation for P{period} - please press Submit or Abort button")

        # always stores OB and pop them if everything is ok or later selecting containers in the tree
        self.pushOB(ob)
        self.tryLastOB()

    def tryLastOB(self):
        self.showOBConfirmationButtons()
        # stop if no ob has to be handled
        if not self.hasOB():
            return

        ob = self.ob_to_confirm[-1]
        # run checkOB which may raise some error before connection request
        # TODO we should test much more because lot of stuff still are beeing processed on live ob creations

        period = ob.getPeriod()
        instrument = self.getInstrument(ob.instrumentConfiguration.name)
        insname = instrument.getShortName()

        # performs operation
        if not self.isConnected():
            self.ui.showLoginFrame(ob)
        elif not self.isReadyToSubmit(ob):
            # self.a2p2client.ui.addToLog("Receive OB for
            # '"+ob.instrumentConfiguration.name+"'")
            self.ui.addToLog(
                f"Selected folder must not be a concatenation. please choose another container for '{insname}' P'{period}'.")
        elif  insname.lower() != self.containerInfo.getInstrument().lower():
            self.ui.addToLog(
                f"'{self.containerInfo.getInstrument()}' selection in not applicable for received OB. Please choose another one for '{insname}' P'{period}'.")
        elif  period != int(self.containerInfo.getIpVersion()):
            self.ui.addToLog(
                f"Container's IpVersion '{int(self.containerInfo.getIpVersion())}' is not applicable for received OB's one '{period}', please select another '{insname}' container or change it in Aspro2.")
        else:
            self.showOBConfirmationButtons(ob, insname=="GRAVITY")

    def showOBConfirmationButtons(self, ob=None, showAcqButtons=False):
        self.ui.showOBConfirmationButtons(ob, showAcqButtons)

    def isReadyToSubmit(self, ob):
        return self.api and self.containerInfo.isOk(ob)

    def isConnected(self):
        return self.connected

    def setConnected(self, flag):
        self.connected = flag

    def pushOB(self,ob):
        self.ob_to_confirm.append(ob)

    def hasOB(self):
        return len(self.ob_to_confirm)>0

    def popOB(self, ignore=False):
        """ Called as last step : ob is for the selected container and User click on submit!
        If ignore is set to True, last ob is simply removed else it is forwarded to P2 and removed from stack. """

        if not self.hasOB():
            return

        # pop last OB
        o = self.ob_to_confirm.pop()
        self.showOBConfirmationButtons()

        if ignore:
            self.ui.addToLog("Last received OB has been aborted")
            return
        else:
            self.ob_to_submit.append(o)

    def processLastAcceptedOB(self):
        if len(self.ob_to_submit)==0:
            return

        o=self.ob_to_submit.pop()
        try:
            instrument = self.getInstrument(o.instrumentConfiguration.name)
            instrument.checkOB(o)
            # check for warnings and abort if requested by user
            # this implementation should be more detailled so we can have various VERSION associated to various warning
            if len(instrument.warnings)>0:
                linesep="\n  -  "
                header=""
                try:
                    # some ditTable may have a VERSION entry some not...
                    version=instrument.getDitTable()["VERSION"]
                    header= f"According to the template manual {version} :\n \n  - \n"
                except:
                    pass
                answer = self.ui.AskYesNoMessage(f"{header} {linesep.join(instrument.warnings)}\n\nClick \"Yes\" to proceed \"No\" to abort the submission")
                instrument.warnings.clear()
                if answer == False:
                    self.ui.addToLog("Submission has been marked to be aborted - do not proceed")
                    return

            self.ui.addToLog("Everything ready! Request OB creation inside selected container")
            instrument.submitOB(o, self.containerInfo)

            # TODO add P2Error handling P2Error(r.status_code, method, url,
            # r.json()['error'])
        except ValueError as e:
            traceback.print_exc()
            trace = traceback.format_exc(limit=1)
            # ui.ShowErrorMessage("Value error :\n %s \n%s\n\n%s" % (e, trace,
            # "Aborting submission to P2. Look at the whole traceback in the log."))
            self.ui.ShowErrorMessage("Value error :\n %s \n\n%s" %
                                    (e, "Aborting submission to P2. Please check LOG and fix before new submission."))
            trace = traceback.format_exc()
            self.ui.addToLog(trace, False)
            self.ui.setProgress(0)
        except Exception as e:
            traceback.print_exc()
            # limit = 2 should raise errors in our codes
            trace = traceback.format_exc(limit=1)
            self.ui.ShowErrorMessage(
                "General error or Absent Parameter in template!\n Missing magnitude or OB not set ?\n\nError :\n %s \n Please check LOG and fix before new submission." % (
                    trace))
            trace = traceback.format_exc()
            self.ui.addToLog(trace, False)
            self.ui.setProgress(0)

    def getStatus(self):
        ob_info=""
        if self.hasOB():
            ob_info=f" / {len(self.ob_to_confirm)} OB in stack"
        if self.isConnected():
            return f" P2API({self.apitype}) connected with {self.username} { ob_info }"

    def connectAPI(self, username, password, ob):
        if username == TUTORIAL_LOGIN: # or username == JMMC_LOGIN:
            self.apitype = DEMO_APITYPE
        else:
            self.apitype = PRODUCTION_APITYPE
        try:
            logger.info("Connecting to p2 api...")
            self.api = p2api.ApiConnection(self.apitype, username, password)

            self.ui.addToLog("Connected to p2 api (%s)"%self.apitype)
            # TODO test that api is ok and handle error if any...

            self.refreshTree()

            self.setConnected(True)
            self.username = username
            self.ui.showTreeFrame(ob)
        except:
            self.ui.ShowErrorMessage("Can't connect to P2 (see LOG).")
            trace = traceback.format_exc()
            self.ui.addToLog(trace, False, level=logging.ERROR)

    def refreshTree(self):
        runs, _ = self.api.getRuns()
        self.ui.fillTree(runs)

    def isDemoAPI(self):
        return self.apitype == DEMO_APITYPE

    def isTutorialAccount(self):
        return self.username == TUTORIAL_LOGIN

    def getAPI(self):
        return self.api

    def getConfDir(self):
        """
        returns the configuration directory with instrument's json files
        """
        return CONFDIR

    def getIssVltiTypes(self):
        return ['snapshot','imaging','time-series','astrometry']



# TODO Move code out of this class

class P2Container:
    # run object stores:
    # {'pi': {'lastName': 'Accont', 'emailAddress': '52052@nodomain.net', 'firstName': 'Phase 1/2 Ttorial'},
    # 'telescope': 'VLTI', 'title': 'p2 tutorial', 'schedledPeriod': 60, 'isToO': False, 'period': 60, 'owned': Tre,
    # 'ipVersion': 104.05, 'instrument': 'PIONIER', 'containerId': 1601182, 'observingConstraints': {'fli' : 'd', 'seeing': 0.8},
    # 'mode': 'SM', 'progId': '60.A-9253(T)', 'itemCont': 2, 'delegated': False, 'runId': 60925319}
    # and item (when not run) :
    # {u'containerStatus': u'P', u'itemType': u'Group', u'name': u'New Group', u'groupScore': 0.0,
    # u'containerId': 2609528, u'runId': 60925213, u'userPriority': 1, u'itemCount': 0, u'parentContainerId': 2609516}

    def __init__(self, facility):
        self.facility = facility
        self.run = None  # dict returned by p2api
        self.containerId = None
        self.item = None

    def store(self, run, item):
        self.run = run
        self.item = item
        self.containerId = item['containerId']
        self.log()

    def reset(self):
        self.run = None  # dict returned by p2api
        self.containerId = None
        self.item = None

    def log(self):
        logmsg = f"*** Working with {self} ***"
        if self.run != self.item and self.item['itemType'] == ITEMTYPE_CONCATENATION:
            self.reset()
            self.facility.ui.addToLog(
                "*** Please do not select a Concatenation and select another container. Or just append a calibrator. ***", )
            logger.info(logmsg)
        else:
            self.facility.ui.addToLog(logmsg)

    def isOk(self, ob):
        """ Tell if given container is ready to receive content of given OB."""
        if self.run == None:
            return False
        if self.run == self.item:
            return True

        has_calib = False
        has_science = False
        for obsConf in ob.observationConfiguration:
            if 'SCIENCE' in obsConf.type:
                has_science = True
            else:
                has_calib = True

        if has_science:
            return self.item['itemType'] != ITEMTYPE_CONCATENATION
        return has_calib

    def getInstrument(self):
        return self.run['instrument']

    def getIpVersion(self):
        return self.run['ipVersion']

    def isRoot(self):
        return self.item.keys() != None and 'pi' in self.item.keys()

    def isServiceModeRun(self):
        return self.run['mode'] == MODE_SM

    def __str__(self):
        return """instrument:'%s', containerId:'%s'""" % (self.run['instrument'], self.containerId)




#!/usr/bin/env python

__all__ = []

import re

import numpy as np

from a2p2.vlti.instrument import OBConstraints
from a2p2.vlti.instrument import OBTarget
from a2p2.vlti.instrument import TSF
from a2p2.vlti.instrument import VltiInstrument

HELPTEXT = """
Please define Gravity instrument help in a2p2/vlti/gravity.py
"""


class Gravity(VltiInstrument):

    def __init__(self, facility):
        VltiInstrument.__init__(self, facility, "GRAVITY")

    def checkOB(self, ob, p2container=None):

        ui = self.ui

        instrumentConfiguration = ob.instrumentConfiguration
        BASELINE = ob.interferometerConfiguration.stations
        # Compute tel = UT or AT
        # TODO move code into common part
        if "U" in BASELINE:
            tel = "UT"
        else:
            tel = "AT"

        instrumentMode = instrumentConfiguration.instrumentMode

        if "COMBINED" in instrumentMode:
            ins_spec_pol = "OUT"
        else:
            ins_spec_pol = 'IN'

        for observationConfiguration in self.getSequence(ob):

            # define top level variable
            if 'SCIENCE' in observationConfiguration.type:
                OBJTYPE = 'SCIENCE'
            else:
                OBJTYPE = 'CALIBRATOR'

            wideMode = self.getAcquisitionType() == "wide"
            onaxisMode = self.isCalibrator(OBJTYPE) or self.getAcquisitionType() == "onaxis"
            offaxisMode = self.isScience(OBJTYPE) and self.getAcquisitionType() == "offaxis"

            # skip calib with wide mode
            # According to templat manual P112 : Calibrations of the interferometric transfer function are thus not needed and not offered for this mode
            if OBJTYPE == 'CALIBRATOR' and wideMode:
                ui.addToLog("Calibrator ignored with wide mode")
            else:

                # dualField is true if FT Target is defined, else false
                ftTarget = ob.get(observationConfiguration, "FTTarget")
                if ftTarget:
                    dualField = True
                else:
                    dualField = False

                # create keywords storage objects
                acqTSF = TSF(self, self.getGravityAcqTemplateName(
                    ob, OBJTYPE, dualField))
                obsTSF = TSF(self, self.getGravityObsTemplateName(
                    ob, OBJTYPE, dualField))

                obConstraints = OBConstraints(self)

                # Check SPEC_RES from instrumentMode and set SPEC_POL except for single_offaxis ( missing keyword INS_SPEC_RES)
                if offaxisMode and not dualField:
                    for res in ("LOW","MED","HIGH"):
                        if res in instrumentMode[0:len(res)]:
                            ins_spec_res = res
                else:
                    ins_spec_res = instrumentMode
                    for res in self.getRange(acqTSF.getName(), "INS.SPEC.RES"):
                        if res in instrumentMode[0:len(res)]:
                            ins_spec_res = res
                    acqTSF.INS_SPEC_RES = ins_spec_res
                    acqTSF.INS_SPEC_POL = ins_spec_pol

                acqTSF.INS_FT_POL = ins_spec_pol

                scienceTarget = observationConfiguration.SCTarget

                obTarget = OBTarget(self, scienceTarget)

                # define target
                acqTSF.SEQ_INS_SOBJ_NAME = obTarget.name

                # Set baseline  interferometric array code (should be a keywordlist)
                acqTSF.ISS_BASELINE = [self.getBaselineCode(ob)]

                self.checkIssVltiType(acqTSF)

                # define some default values
                acqTSF.SEQ_INS_SOBJ_DIAMETER = float(
                    self.get(scienceTarget, "DIAMETER", 0.0))

                # Retrieve Fluxes
                acqTSF.COU_GS_MAG = self.getFlux(scienceTarget, "V")
                acqTSF.SEQ_INS_SOBJ_MAG = self.getFlux(scienceTarget, "K")
                if not dualField or wideMode:
                    acqTSF.SEQ_INS_SOBJ_HMAG = self.getFlux(scienceTarget, "H")

                # setup some default values, to be changed below
                GSRA = '00:00:00.000'
                GSDEC = '00:00:00.000'

                # if FT Target is not ScTarget, we are in dual-field (TBD)
                ftTarget = ob.get(observationConfiguration, "FTTarget")
                if ftTarget != None:
                    acqTSF.SEQ_FT_ROBJ_NAME = ftTarget.name
                    acqTSF.SEQ_FT_ROBJ_MAG = self.getFlux(ftTarget, "K")
                    acqTSF.SEQ_FT_ROBJ_HMAG = self.getFlux(ftTarget, "H")

                    # test arcseconds distance in dual field or wide mode (GRAVITY_TemplateManual_P112 values)
                    if tel == "UT":
                        if wideMode:
                            SCtoREFmaxDist = 30
                            SCtoREFminDist = 2
                        elif offaxisMode:
                            SCtoREFmaxDist = 2
                            SCtoREFminDist = .27
                        else:
                            SCtoREFmaxDist = .6
                            SCtoREFminDist = 0
                    else:
                        if wideMode:
                            SCtoREFmaxDist = 30
                            SCtoREFminDist = 4
                        elif offaxisMode:
                            SCtoREFmaxDist = 4
                            SCtoREFminDist = 1.17
                        else:
                            SCtoREFmaxDist = 2.7
                            SCtoREFminDist = 0

                    FTRA, FTDEC = self.getCoords(ftTarget)
                    dualFieldSeparation = self.getSkySeparation(obTarget.ra, obTarget.dec, FTRA, FTDEC)
                    dualFieldSeparation = dualFieldSeparation.arcsec
                    if dualFieldSeparation <= SCtoREFminDist or dualFieldSeparation >= SCtoREFmaxDist:
                        raise ValueError(f"Dual-Field separation of two stars {dualFieldSeparation:.3f}'' is out of range [{SCtoREFminDist}'', {SCtoREFmaxDist}''], Please correct.")
                    else:
                        ui.addToLog(f"Dual-Field separation of two stars is {dualFieldSeparation:.3f}''")

                    # Provide FT position
                    if wideMode:
                        acqTSF.SEQ_FT_ROBJ_ALPHA = FTRA
                        acqTSF.SEQ_FT_ROBJ_DELTA = FTDEC
                    else:
                        # compute x,y between science and ref beams and set X,Y (mas) keywords:
                        dualFieldOffset = self.getSkyOffset(obTarget.ra, obTarget.dec, FTRA, FTDEC)
                        acqTSF.SEQ_INS_SOBJ_X = dualFieldOffset[0].mas
                        acqTSF.SEQ_INS_SOBJ_Y = dualFieldOffset[1].mas

                    FTPMA, FTPMD = self.getPMCoords(ftTarget)
                    acqTSF.SEQ_FT_ROBJ_PMA = FTPMA
                    acqTSF.SEQ_FT_ROBJ_PMD = FTPMD

                    FTPARALLAX = self.getPARALLAX(ftTarget)
                    acqTSF.SEQ_FT_ROBJ_PARALLAX = FTPARALLAX

                    acqTSF.SEQ_FT_MODE = "AUTO"

                # AO target
                aoTarget = ob.get(observationConfiguration, "AOTarget")
                if aoTarget != None:
                    AONAME = aoTarget.name
                    acqTSF.COU_AG_GSSOURCE = 'SETUPFILE'  # since we have an AO
                    acqTSF.COU_AG_ALPHA, acqTSF.COU_AG_DELTA = self.getCoords(
                        aoTarget, requirePrecision=False)
                    acqTSF.COU_AG_PMA, acqTSF.COU_AG_PMD = self.getPMCoords(
                        aoTarget)
                    # Case of CIAO to be implemented...based on v and k magnitudes?
                    acqTSF.COU_GS_MAG = float(aoTarget.FLUX_V)

                # Guide Star
                gsTarget = ob.get(observationConfiguration, 'GSTarget')
                if gsTarget != None and aoTarget == None:
                    acqTSF.COU_AG_SOURCE = 'SETUPFILE'  # since we have an GS
                    acqTSF.COU_AG_ALPHA, acqTSF.COU_AG_DELTA = self.getCoords(
                        gsTarget, requirePrecision=False)
                    acqTSF.COU_AG_PMA, acqTSF.COU_AG_PMD = self.getPMCoords(
                        gsTarget)
                    acqTSF.COU_GS_MAG = float(gsTarget.FLUX_V)

                # LST interval
                try:
                    obsConstraint = observationConfiguration.observationConstraints
                    LSTINTERVAL = obsConstraint.LSTinterval
                except:
                    LSTINTERVAL = None

                # Constraints
                obConstraints.name = 'Aspro-created constraints'
                skyTransparencyMagLimits = {"AT": 3, "UT": 5}
                if acqTSF.SEQ_INS_SOBJ_MAG < skyTransparencyMagLimits[tel]:
                    obConstraints.skyTransparency = 'Variable, thin cirrus'
                else:
                    obConstraints.skyTransparency = 'Clear'
                # FIXME: error (OB): "Phase 2 constraints must closely follow what was requested in the Phase 1 proposal.

                # The seeing value allowed for this OB is >= java0x0 arcsec."
                # FIXME REPLACE SEEING THAT IS NO MORE SUPPORTED
                # obConstraints.seeing = 1.0

                # FIXME: default values NOT IN ASPRO!
                # constaints.airmass = 5.0
                # constaints.fli = 1



                # compute dit, ndit, nexp
                dit = self.getDit(tel, ins_spec_res, ins_spec_pol,
                                acqTSF.SEQ_INS_SOBJ_MAG, dualField, targetName=scienceTarget.name.strip(),  showWarning=(p2container == None))
                ndit = 300 / dit
                if ndit < 10:
                    ndit = 10
                if ndit > 300:
                    ndit = 300
                ndit = int(ndit)  # must be integer
                exptime = int(ndit * dit + 40)  # 40 sec overhead by exp
                nexp = (1800 - 900) / exptime
                nexp = int(nexp)
                ui.addToLog(
                    'number of exposures to reach 1800 s per OB is ' + str(nexp))
                if nexp < 3:
                    nexp = 3  # min is O S O
                    # recompute ndit
                    exptime = (1800 - 900) / nexp
                    ndit = (exptime - 40) / dit
                    ndit = int(ndit)
                    if ndit < 10:
                        ndit = 10
                        ui.addToLog(
                            "**Warning**, OB NDIT has been set to min value=%d, but OB will take longer than 1800 s" % (
                                ndit))
                nexp %= 40
                sequence = 'O S O O S O O S O O S O O S O O S O O S O O S O O S O O S O O S O O S O O S O O'
                my_sequence = sequence[0:2 * nexp]
                # and store computed values in obsTSF
                obsTSF.DET2_DIT = str(dit)
                obsTSF.DET2_NDIT_OBJECT = ndit
                obsTSF.DET2_NDIT_SKY = ndit
                obsTSF.SEQ_OBSSEQ = my_sequence

                obsTSF.SEQ_SKY_X = 2000
                obsTSF.SEQ_SKY_Y = 2000

                # then call the ob-creation using the API if p2container exists.
                if p2container == None:
                    ui.addToLog(obTarget.name +
                                " ready for p2 upload (details logged)")
                    ui.addToLog(obTarget, False)
                    ui.addToLog(obConstraints, False)
                    ui.addToLog(acqTSF, False)
                    ui.addToLog(obsTSF, False)
                else:
                    self.createOB(p2container, obTarget, obConstraints, OBJTYPE, instrumentMode,
                                  LSTINTERVAL, [acqTSF, obsTSF])

    def formatDitTable(self):
        ditTable = self.getDitTable()
        buffer = '    Mode     |Spec |  Pol  |Tel |       K       | DIT(s)\n'
        buffer += '--------------------------------------------------------\n'
        for tel in ['AT']:
            for spec in ['LOW', 'MED', 'HIGH']:
                for pol in ['OUT', 'IN']:
                    for i in range(len(ditTable[tel][spec][pol]['DIT'])):
                        buffer += 'Single Field | %4s | %3s | %2s |' % (
                            spec, pol, tel)
                        buffer += ' %4.1f <K<= %3.1f | %4.1f' % (ditTable[tel][spec][pol]['MAG'][i],
                                                                 ditTable[tel][spec][
                                                                     pol]['MAG'][i + 1],
                                                                 ditTable[tel][spec][pol]['DIT'][i])
                        buffer += "\n"
            Kdf = ditTable[tel]['Kdf']
            Kut = ditTable[tel]['Kut']
            buffer += ' Dual Field  |  all | all | %2s | Kdf = K - %.1f |  -\n' % (
                tel, Kdf)
            tel = "UT"
            buffer += 'Single Field |  all | all | %2s | Kdf = K - %.1f |  -\n' % (
                tel, Kut)
            buffer += ' Dual Field  |  all | all | %2s | Kdf = K - %.1f |  -\n' % (
                tel, Kut + Kdf)
        return buffer

    def getGravityObsTemplateName(self, ob, OBJTYPE, dualField):
        return self.getGravityTemplateName("obs", ob, OBJTYPE, dualField)

    def getGravityAcqTemplateName(self, ob, OBJTYPE, dualField):
        return self.getGravityTemplateName("acq", ob, OBJTYPE, dualField)

    def getGravityTemplateName(self, templateType, ob, OBJTYPE, dualField):

        dualValue = {True: "dual", False: "single"}
        objValue = {"SCIENCE": "exp", "CALIBRATOR": "calibrator"}

        # Add : name , <dual|single>, <obs|acq>, [objValue]
        items = [self.getName()]
        # Add dual or single for SCIENCE or single for other OBJTYPES
        items.append(dualValue[dualField and self.isScience(OBJTYPE)])
        # Add acquisitionType (or onaxis for CAL) after P112 if acq
        if templateType == "acq" and ob.getPeriod() >= 112:
            if self.isScience(OBJTYPE):
                items.append(self.getAcquisitionType())
            else:
                items.append("onaxis")
        # Add acq or obs
        items.append(templateType)
        # Add objValue if obs
        if templateType == "obs":
            items.append(objValue[OBJTYPE])

        return "_".join([i for i in items if i is not None])+".tsf"



#!/usr/bin/env python
from __future__ import with_statement

import traceback

from argparse import ArgumentParser
from a2p2 import __release_notes__
from distutils.version import LooseVersion



def createMdReleaseNotes():
    """
    Present in a reverse ordered way the dictionnary version items.
    """
    txt = "# A2P2 release notes"
    lvs = [LooseVersion(v) for v in __release_notes__]
    for lv in sorted(lvs, reverse=True):
        v=lv.vstring
        txt += "\n## V " + v + " :"
        keys = []
        for k in sorted(__release_notes__[v]):
            if not ("TODO" in k):
                keys.append(k)
        for k in sorted(__release_notes__[v]):
            if "TODO" in k:
                keys.append(k)

        for e in keys:
            infos = __release_notes__[v][e]
            if infos:
                txt += "\n### " + e + " : "
                for i in infos:
                    txt += "\n- " + i
        txt += "\n\n"
    f=open("release-notes.md", "w")
    f.write(txt)
    print(f"{f.name} generated")


def main():
    """Main method to start a2p2 program."""
    parser = ArgumentParser(
        description='Move your Aspro2 observation details to an observatory proposal database')
    # parser.add_argument('-c', '--config', action='store_true', help='show instruments and remote service configurations.')
    parser.add_argument('-f', '--fakeapi', action='store_true',
                        help='fake API to avoid remote connection (dev. only).')
    parser.add_argument('-v', '--verbose', action='store_true', help='Verbose')
    parser.add_argument('-c', '--createprefs', action='store_true', help='Create preferences file')
    parser.add_argument('-r', '--releasenotes', action='store_true', help='Create md release notes file')

    args = parser.parse_args()

    from . import A2p2Client

    if args.releasenotes:
        createMdReleaseNotes()
        exit()

    if args.createprefs:
        A2p2Client.createPreferencesFile()
        exit()


    try:
        with A2p2Client(args.fakeapi, args.verbose) as a2p2c:
            # if  args.config:
            #    print(a2p2c)
            # else:
            #    a2p2c.run()
            a2p2c.run()

    except Exception as e:
        if True or args.verbose:
            traceback.print_exc()
        else:
            print('ERROR: %s' % str(e))


if __name__ == '__main__':
    main()
